(function($) {
	$.fn.visible = function(partial, hidden) {
		var $t = $(this).eq(0),
			t = $t.get(0),
			$w = $(window),
			viewTop = $w.scrollTop(),
			viewBottom = viewTop + $w.height(),
			_top = $t.offset().top,
			_bottom = _top + $t.height(),
			compareTop = _top,
			compareBottom = _bottom,
			clientSize = true;
		return !!clientSize && (compareBottom <= viewBottom && compareTop >= viewTop)
	};
	var table, searchFilter, elementsTemplate, allData;
	$.fn.commandPalette = function(methodOrOptions) {
		var filteredData, onItemSelected, onFilter;

		var methods = {
			init: function(options) {
				this.empty();
				elementsTemplate = options.elementsTemplate;
				onItemSelected = options.onItemSelected;
				onFilter = options.onFilter;
				allData = filteredData = options.items || [];
				if (elementsTemplate) {
					var compiledTemplate = tmpl(elementsTemplate);
					compileTemplateFunction = function(item) {
						item.tmpl = '<div id="item-' + item.index + '">' + compiledTemplate(item) + "</div>"
					}
				} else {
					compileTemplateFunction = function(item) {
						item.tmpl = '<div id="item-' + item.index + '">$$string$$</div>'
					}
				}
				for (var i = 0, li = allData.length; i < li; i++) {
					allData[i].index = i;
					compileTemplateFunction(allData[i])
				}
				this.append('<input type="text" id="searchFilter" class="searchFilter theme-text-color-inverse theme-background-color-mark" placeholder=""><div id="results" class="results palette-results theme-text-color-inverse theme-background-color-mark"></div>');
				table = this.find(".results");
				searchFilter = this.find(".searchFilter").focus();


				updateTableDOM();



				document.getElementById('command-palette-wrapper').onclick = function(event) {

					// click on child hack
					if (event.toElement.parentElement.id.substring(0, 5) === "item-") {
						event.toElement.parentElement.click();
					}

					if (event.toElement.id.substring(0, 5) === "item-") {

						var selected = table.find(".selected");
						if (selected[0].id != event.srcElement.id) {
							$(event.toElement).addClass("selected theme-background-color-mark-modified");
							selected.removeClass("selected theme-background-color-mark-modified");
						}
						var selectedIndex = parseInt(event.toElement.id.substring(5), 10);
						onItemSelected(allData[selectedIndex]);
						$("#searchFilter").focus();
					}
				};

				searchFilter.keydown(function(event) {

					var keyCode = event.keyCode;
					if (keyCode == "38") {
						var selected = table.find(".selected");
						var prev = selected.prev();
						if (prev.length) {
							prev.addClass("selected theme-background-color-mark-modified");
							selected.removeClass("selected theme-background-color-mark-modified");
							//if (!prev.visible()) { // buggy
							//prev.get(0).scrollIntoView();
							//}
							prev.get(0).scrollIntoViewIfNeeded(false);

						}
					} else if (keyCode == "40") {
						var selected = table.find(".selected");
						var next = selected.next();
						if (next.length) {
							next.addClass("selected theme-background-color-mark-modified");
							selected.removeClass("selected theme-background-color-mark-modified");
							//if (!next.visible()) { // buggy
							//next.get(0).scrollIntoView(false);
							//}
							next.get(0).scrollIntoViewIfNeeded(false);

						}
					} else if (keyCode == "13") {
						var selectedIndex = parseInt(table.find(".selected")[0].id.substring(5), 10);
						onItemSelected(allData[selectedIndex])
					} else {
						fuzzySearch()
					}
				})
			},
			update: function(items) {
				allData = filteredData = items || [];
				if (elementsTemplate) {
					var compiledTemplate = tmpl(elementsTemplate);
					compileTemplateFunction = function(item) {
						item.tmpl = '<div id="item-' + item.index + '">' + compiledTemplate(item) + "</div>"
					}
				} else {
					compileTemplateFunction = function(item) {
						item.tmpl = '<div id="item-' + item.index + '">$$string$$</div>'
					}
				}



				for (var i = 0, li = allData.length; i < li; i++) {
					allData[i].index = i;
					compileTemplateFunction(allData[i]);
				}



				updateTableDOM()
			}
		};
		if (methods[methodOrOptions]) {
			return methods[methodOrOptions].apply(this, Array.prototype.slice.call(arguments, 1))
		} else if (typeof methodOrOptions === "object" || !methodOrOptions) {
			return methods.init.apply(this, arguments)
		} else {
			$.error("Method " + methodOrOptions + " does not exist on jQuery.commandPalette")
		}

		function createResultElement(item) {
			var tmpl = item.tmpl || item.original.tmpl;
			var shortcut = (item.original) ? item.original.shortcut : item.shortcut;

			var type = (item.original) ? item.original.type : item.type;
			var sublist = (item.original) ? item.original.sublist : item.sublist;
			
			if (type == "sublist" || sublist.length > 0) var smark = "&nbsp;&nbsp;&#xf149;"
			else var smark = "";

			var widgetObject = (item.original) ? item.original.widget : item.widget;
			var widget = widgetObject != null ? getWidget(widgetObject) : "";
			
			var persist = (item.original) ? item.original.persist : item.persist;
			persist = persist === true ? "&nbsp;&nbsp;&#xf177;" : "";
			
			var icon = (item.original) ? item.original.icon : item.icon;
			icon = icon !== undefined ? icon : "";
			icon = icon.startsWith("&#") ? '<span class="awf" style="padding-left: 1px; margin-right: .5em; opacity: .6;">' + icon + '</span>' : icon;


			//var itemStr = (item.original) ? item.original.string : item.string;
			var itemRecent = (item.original) ? item.original.recent : item.recent;
			itemRecent = (itemRecent) ? itemRecent : "";
			var itemStr = itemRecent.startsWith("0") ? '<span class="palette-recent theme-border-color-inverse">' + item.string + '</span>' : item.string;

			

			return $(tmpl.replace("$$ico$$ ", icon).replace("$$string$$", itemStr).replace("$$shortcut$$", shortcut).replace("$$sub$$", smark).replace("$$widget$$", widget).replace("$$persist$$", persist));
		}


		function getWidget(w) {
			var wString = "";
			
			if (w !== "") {
				switch (w.type) {
					case "setting":
						wString = localStorage[w.value] == "1" ? "ON" : "OFF";
						break;
					case "eval":
						wString = eval(w.value);
						break;
					default:
						wString = "";
				}
				return '<span class="textWidget theme-background-color theme-mark-color">' + wString + '</span>';

			} else { // no widget, return empty string
				return "";
			}
		}


		function updateTableDOM() {
			table.empty();
			var fragment = document.createDocumentFragment();
			if (filteredData.length) {

				var el = createResultElement(filteredData[0]).addClass("selected theme-background-color-mark-modified");
				fragment.appendChild(el.get(0));

				for (var i = 1, li = filteredData.length; i < li; i++) {
					var el = createResultElement(filteredData[i]);
					fragment.appendChild(el.get(0));
				}

			}
			table.get(0).appendChild(fragment);

		}

		function fuzzySearch() {
			setTimeout(function() {
				var fuzzySearchOptions = {
					pre: "<b>",
					post: "</b>",
					extract: function(el) {
						return el.string
					}
				};
				filteredData = fuzzy.filter(searchFilter[0].value, allData, fuzzySearchOptions);
				if (onFilter) {
					onFilter(searchFilter[0].value, filteredData, $(".selected").get(0));
				}
				updateTableDOM();
			}, 1)
		}
		return this
	}
})(jQuery);
(function() {
	var cache = {};
	this.tmpl = function tmpl(str, data) {
		var fn = !/\W/.test(str) ? cache[str] = cache[str] || tmpl(document.getElementById(str).innerHTML) : new Function("obj", "var p=[],print=function(){p.push.apply(p,arguments);};" + "with(obj){p.push('" + str.replace(/[\r\t\n]/g, " ").split("{{").join("	").replace(/((^|\}\})[^\t]*)'/g, "$1\r").replace(/\t=(.*?)\}\}/g, "',$1,'").split("	").join("');").split("}}").join("p.push('").split("\r").join("\\'") + "');}return p.join('');");
		return data ? fn(data) : fn
	}
})();