var plugins = (function() {


	// get rooth application path
	// this should be relative!!!!
	// var pluginsRoot = path.dirname(process.execPath) + "/plugins";
	var pluginsRoot = "plugins";



	function __init() {

	}


	// get array of valid plugin info object / if valid package.json file is found
	// also check for mandatory id and main fileds
	function getInstalledPluginsInfo() {

		var pluginDirs = fs.readdirSync(pluginsRoot).filter(function(file) {
			return fs.statSync(path.join(pluginsRoot, file)).isDirectory();
		});


		// get plugins info
		var pluginsInfo = [];
		var registeredIdsHelper = [];

		for (var i = 0; i < pluginDirs.length; i++) {
			var pluginPath = pluginsRoot + "/" + pluginDirs[i];

			var packagePath = pluginPath + "/package.json";

			try {
				var infoString = fs.readFileSync(packagePath, {
					encoding: "utf-8"
				});


				var infoObject = JSON.parse(infoString);

				// check if valid package
				if (!infoObject.id || !infoObject.main || infoObject.main == "" || infoObject.id == "") {
					console.log("PLUGINS - " + pluginDirs[i] + " is not a valid plugin!");
					continue;
				}

				// check if id already exists
				else if (registeredIdsHelper.indexOf(infoObject.id) > -1) {
					console.log("PLUGINS - " + pluginDirs[i] + " - plugin with the same id is already running!");
					continue;
				}

				// check if enabled
				else if (!infoObject.enabled) {
					console.log("PLUGINS - " + pluginDirs[i] + " is disabled and will not run!");
					continue;
				}

				// check if compatible
				else if (!infoObject.dependencies || !infoObject.dependencies.wm || !isPluginCompatible(infoObject.dependencies.wm)) {
					console.log("PLUGINS - " + pluginDirs[i] + " is not compatible with this version of wm!");
					continue;
				}

				// add executable path into object
				infoObject["main_path"] = "./" + pluginPath + "/" + infoObject.main;

				// add plugin dir path
				infoObject["plugin_dir_path"] = "./" + pluginPath;

				// add plugin absolute dir path
				infoObject["plugin_dir_path_abs"] = path.resolve("./") + "/" + pluginPath;




				//push into array
				pluginsInfo.push(infoObject);
				registeredIdsHelper.push(infoObject.id);

			} catch (e) {
				// no package.json file, skip this directory / mot a plugin
				console.log("PLUGINS - " + pluginDirs[i] + " - Error executing plugin! ... " + e);
				continue;
			}

		}

		return pluginsInfo;

	}






	function runInstalledPlugins() {
		if (wm.ckk() == -1) return;
		var pluginsInfo = getInstalledPluginsInfo();
		for (var i = 0; i < pluginsInfo.length; i++) {
			activatePlugin(pluginsInfo[i]);
		}
	}


	function activatePlugin(pObj) {

		// run plugin
		try {
			_plugins[pObj.id] = require(pObj.main_path)(monkeyAPI, pObj); // 3 params to each plugin - api and plugin info
			console.log("PLUGINS - " + pObj.id + " - Loaded!");
		} catch (ex) {
			console.log(ex);
		}

		// inject css if enabled
		if (pObj.css && pObj.css.trim().length > 0) {
			injectCSS(pObj);
		}


		// run init if exists
		if (typeof _plugins[pObj.id]._init === "function") {
			_plugins[pObj.id]._init();
			console.log("PLUGINS - " + pObj.id + " - Initialized!");
		}
	}

	// cals _unplug function
	function deactivatePlugin(pObj) {

	}


	// inject css
	function injectCSS(pObj) {

		var cssPath = pObj.plugin_dir_path + "/" + pObj.css;

		fs.readFile(cssPath, function(err, data) { // should this be sync instead??
			if (err) {
				console.log("PLUGINS - " + pObj.id + " - file: " + pObj.css + " not found!");
			} else {
				var cssString = data.toString();
				var head = document.getElementsByTagName('head')[0];
				var style = document.createElement("style");
				style.appendChild(document.createTextNode(cssString));
				head.appendChild(style);
				console.log("PLUGINS - " + pObj.id + " - Adding CSS rules!");
			}
		});
	}


	function isPluginRunning(pId) {
		if (typeof _plugins[pId] != "undefined") {
			return true;
		 } else {
			 return false;
		 }
	}


	// check if plugin is compatible with installed version
	// DO NOT GO OVER 9 FOR EACH PART!!!!!!!!!!
	function isPluginCompatible(pluginVersion) {
		var appVersion = app.manifest.version;

		var appVersionNum = parseInt(appVersion.split('.').join(''), 10);
		var pluginVersionNum = parseInt(pluginVersion.split('.').join(''), 10);

		if (appVersionNum < pluginVersionNum)
			return false;
		else
			return true;

		// for (var i = 0; i < appVersionParts.length; i++) {
		// 	var appN = parseInt(appVersionParts[i], 10);
		// 	var plgN = parseInt(pluginVersionParts[i], 10);
		// 	if (appN < plgN) return false;
		// }
		// return true;
	}


	return {
		activatePlugin: activatePlugin,
		deactivatePlugin: deactivatePlugin,
		runInstalledPlugins: runInstalledPlugins,
		isPluginRunning: isPluginRunning
	};

})();




/*
########   ##          ######            ###     ########   ####
##     ##  ##         ##    ##          ## ##    ##     ##   ##
##     ##  ##         ##               ##   ##   ##     ##   ##
########   ##         ##   ####       ##     ##  ########    ##
##         ##         ##    ##        #########  ##          ##
##         ##         ##    ##        ##     ##  ##          ##
##         ########    ######         ##     ##  ##         ####
*/


var monkeyAPI = (function(window) {

	// API



	return {
		monkey: window,
		window: window,
		editor: window._editor,
		base: window.wm,
		storage: window.storage,
		commandPalette: window.commandPalette,
		localStorage: localStorage

	};

})(window);