module.exports = function(api, pluginInfo) {

	var monkey = window;
	var options = pluginInfo.options;
	var $ = api.window.$;

	var $iframe;
	var paneHandle;
	var templateCSS = "";
	var wmColorsCSS = "";
	var templateJS = "";

	var numberedHeadingsCSS = "<style type=\"text/css\"> body{counter-reset: section;} h2{counter-reset: sub-section;} h3{counter-reset: composite;} h4{counter-reset: detail;} h5{counter-reset: sub-detail;} h2:before{ counter-increment: section; content: counter(section) \". \"; } h3:before{ counter-increment: sub-section; content: counter(section) \".\" counter(sub-section) \". \"; } h4:before{ counter-increment: composite; content: counter(section) \".\" counter(sub-section) \".\" counter(composite) \". \"; } h5:before{ counter-increment: detail; content: counter(section) \".\" counter(sub-section) \".\" counter(composite) \".\" counter(detail) \". \"; } h6:before{ counter-increment: sub-detail; content: counter(section) \".\" counter(sub-section) \".\" counter(composite) \".\" counter(detail) \".\" counter(sub-detail) \". \"; }</style>";

	// var elasticFontSizeCSS = '<style>html, table, code {font-size: 4vw!important}</style>';

	// var renderer = new monkey.marked.Renderer();
	// monkey.marked.setOptions({
	// 	renderer: renderer,
	// 	gfm: true,
	// 	tables: true,
	// 	breaks: true,
	// 	pedantic: false,
	// 	sanitize: false,
	// 	smartLists: true,
	// 	smartypants: true
	// });



	

	function _init() {
		setPane();
		createIframe();
		setPaletteItems();
		setShortcuts();
	}

	function setPane() {
		paneHandle = monkey.universalPane.registerPane("wm_preview", "Markdown preview", {
			backgroundColor: "#ffffff"
			// width: 400
		});
	}


	function setShortcuts() {
		// Keyboard shortcut
		monkey._editor.on("keydown", function (cm, e) {
			if (e.keyCode == 72 && e.shiftKey && (e.ctrlKey || e.metaKey)) {
				copyCleanHtml();
			}
		});
	}

	function getContextMenu() {
		
		var previewContextMenu = new monkey.gui.Menu({
			'type': 'menubar'
		});

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Select template',
			submenu: getTemplatesMenu()
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Print',
			click: function() {
				print();
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Open in browser',
			click: function() {
				saveTempAndExport("html");
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Open in .doc editor',
			click: function() {
				saveTempAndExport("doc");
			}
		}));


		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Copy as clean HTML',
			click: function() {
				copyCleanHtml();
			}
		}));


		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Update',
			click: function() {
				updatePreview();
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'separator',
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1" ? true : false,
			label: 'Include comments',
			click: function() {
				if (monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] && monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1") {
					monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] = "0";
				} else {
					monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] = "1";
				}
				updatePreview();
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_FORCED_NEW_LINES"] == "1" ? true : false,
			label: 'Include forced empty lines',
			click: function() {
				if (monkey.localStorage["EXPORT_FORCED_NEW_LINES"] && monkey.localStorage["EXPORT_FORCED_NEW_LINES"] == "1") {
					monkey.localStorage["EXPORT_FORCED_NEW_LINES"] = "0";
				} else {
					monkey.localStorage["EXPORT_FORCED_NEW_LINES"] = "1";
				}
				updatePreview();
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_HR_PAGE_BREAKS"] == "1" ? true : false,
			label: 'Force page break at horizontal rule',
			click: function() {
				if (monkey.localStorage["EXPORT_HR_PAGE_BREAKS"] && monkey.localStorage["EXPORT_HR_PAGE_BREAKS"] == "1") {
					monkey.localStorage["EXPORT_HR_PAGE_BREAKS"] = "0";
				} else {
					monkey.localStorage["EXPORT_HR_PAGE_BREAKS"] = "1";
				}
				setColors();
				updatePreview();
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] == "1" ? true : false,
			label: 'Numbered headings',
			click: function() {
				if (monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] && monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] == "1") {
					monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] = "0";
				} else {
					monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] = "1";
				}
				updatePreview();
			}
			
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_SYNC_SCROLLING"] == "1" ? true : false,
			label: 'Synchronize scrolling',
			click: function() {
				if (monkey.localStorage["EXPORT_SYNC_SCROLLING"] && monkey.localStorage["EXPORT_SYNC_SCROLLING"] == "1") {
					monkey.localStorage["EXPORT_SYNC_SCROLLING"] = "0";
				} else {
					monkey.localStorage["EXPORT_SYNC_SCROLLING"] = "1";
				}
				updatePreview();
			}
		}));


		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_ZOOM_PREVIEW"] == "1" ? true : false,
			label: 'Auto zoom',
			click: function() {
				if (monkey.localStorage["EXPORT_ZOOM_PREVIEW"] && monkey.localStorage["EXPORT_ZOOM_PREVIEW"] == "1") {
					monkey.localStorage["EXPORT_ZOOM_PREVIEW"] = "0";
				} else {
					monkey.localStorage["EXPORT_ZOOM_PREVIEW"] = "1";
				}
				setColors();
				updatePreview();
			}
			
		}));

		return previewContextMenu;
	}


	function getTemplatesMenu() {

		var templatesSubmenu = new monkey.gui.Menu({
			'type': 'menubar'
		});


		var templatesA = monkey.wm.getInstalledTemplateNames();

		for (var i = 0; i < templatesA.length; i++) {

			// exclude non css files
			if (!templatesA[i].endsWith(".css")) continue;

			var tName = templatesA[i].replace(/\..*$/, "");

			var mItem = new monkey.gui.MenuItem({
				type: 'checkbox',
				checked: monkey.localStorage['EXPORT_TEMPLATE'] == templatesA[i] ? true : false,
				label: tName,
				click: function() {
					changeTemplate(this.label + ".css");
				}
			});

			templatesSubmenu.append(mItem);

		}

		return templatesSubmenu;
	}

	


	function setPaletteItems() {


		// set theme selecton
		monkey.commandPalette.registerPaletteItem({
			id: 'preview-set-template',
			string: 'Markdown Preview: Set Export Template',
			shortcut: "",
			action: '',
			type: "sublist",
			sublist: '_plugins["mkdpreview"].getTemplatePaletteItems()',
			persist: false,
			icon: "&#xf1c9;",
			widget: {
				type: "eval",
				value: 'localStorage["EXPORT_TEMPLATE"]'
			}
		});




		// set print command
		monkey.commandPalette.registerPaletteItem({
				id: 'preview-print',
				string: 'Markdown Preview: Export to Printer',
				shortcut: "",
				action: '_plugins["mkdpreview"].print()',
				type: "eval",
				sublist: [],
				icon: "&#xf1c9;",
				persist: false
			}
		);


	
		monkey.commandPalette.registerPaletteItem({
				id: 'preview-export-html',
				string: 'Markdown Preview: Export to Default Browser',
				shortcut: "",
				action: '_plugins["mkdpreview"].saveTempAndExport("html")',
				type: "eval",
				sublist: [],
				icon: "&#xf1c9;",
				persist: false
			}
		);


		monkey.commandPalette.registerPaletteItem({
				id: 'preview-export-doc',
				string: 'Markdown Preview: Export to Word (default .doc editor)',
				shortcut: "",
				action: '_plugins["mkdpreview"].saveTempAndExport("doc")',
				type: "eval",
				sublist: [],
				icon: "&#xf1c9;",
				persist: false
			}
		);

		monkey.commandPalette.registerPaletteItem({
				id: 'preview-copy-html',
				string: 'Markdown Preview: Copy as Clean HTML',
				shortcut: "Ctrl+Shift+H",
				action: '_plugins["mkdpreview"].copyCleanHtml()',
				type: "eval",
				sublist: [],
				icon: "&#xf1c9;",
				persist: false
			}
		);

		monkey.commandPalette.registerPaletteItem({
				id: 'preview-update',
				string: 'Markdown Preview: Update Preview',
				shortcut: "",
				action: '_plugins["mkdpreview"].updatePreview()',
				type: "eval",
				sublist: [],
				icon: "&#xf1c9;",
				persist: false
			}
		);
	}


	// Get palette items for set template
	function getTemplatePaletteItems() {
		var objectsA = [];
		var templatesA = monkey.wm.getInstalledTemplateNames();

		for (var i = 0; i < templatesA.length; i++) {

			// exclude non css files
			if (!templatesA[i].endsWith(".css")) continue;

			var item = {
				string: "Set Template: " + templatesA[i].replace(/\..*$/, ""),
				shortcut: "",
				action: '_plugins["mkdpreview"].changeTemplate("' + templatesA[i] + '")',
				type: "eval",
				sublist: [],
				icon: "&#xf1c5;",
				persist: true
			};
			objectsA.push(item);
		}
		return objectsA;
	}


	// change preview template 
	function changeTemplate(name) {
		monkey.wm.setTemplate(name);
		templateCSS = monkey.wm.getCssFromTemplateFile(name);
		templateJS = monkey.wm.getJsFromTemplateFile(name);
		setColors();
		updatePreview();
	}



	function createIframe() {

		$("#wm_preview").empty();


		$iframe = $('<iframe scrolling="yes" id="preview-iframe" style="margin-left: .3rem; background-color: transparent; position: relative; width: calc(100% - .3rem); height: calc(100vh - 2rem);" nwfaketop nwdisable></iframe>').appendTo("#wm_preview");


		if (monkey.localStorage["EXPORT_TEMPLATE"]) {
			templateCSS = monkey.wm.getCssFromTemplateFile(monkey.localStorage["EXPORT_TEMPLATE"]);
			templateJS = monkey.wm.getJsFromTemplateFile(monkey.localStorage["EXPORT_TEMPLATE"]);
		}
		
		setColors();

	}



	function setColors(isExport) {
		if (typeof isExport === 'undefined') isExport = false;
	
		// bcHex = "#ffffff";		
		// fcHex = "#090909";

		// Insert Auto zoom snippet if needed
		var autoZoom;

		if (isExport) {
			autoZoom = '@media screen {html, body {font-size: 1em!important; max-width: 70%; margin-left: auto; margin-right: auto; }} @media print {html,body {font-size: .9em!important; max-width: 100%; margin-left: 0em; margin-right: 0em;}}';
		} else {
			if (monkey.localStorage["EXPORT_ZOOM_PREVIEW"] == "1") {
				autoZoom = '@media screen {html, body {font-size: ' + options.preview_zoom_ratio + 'vw!important;}} @media print {html,body {font-size: .9em!important;}}';
			} else {
				autoZoom = '@media screen {html, body {font-size: .9em!important;}} @media print {html,body {font-size: .9em!important;}}';
			}
			
		}

		var noLinks = '';
		var pageBreaks = '';
		if (!isExport) noLinks = ' a {pointer-events: none; border-bottom-width: 0px!important; color: inherit !important; text-decoration: underline!important;} ';
		if (monkey.localStorage["EXPORT_HR_PAGE_BREAKS"] == "1") pageBreaks = ' hr {page-break-after: always!important; visibility: hidden!important; padding: 0!important; margin: 0!important; } ';

		var rotate = "1.3";
		
		//(Math.random() * _settings.collapsed_region_comment_max_tilt) - _settings.collapsed_region_comment_max_tilt / 2;
		
		wmColorsCSS = '<style type="text/css">' + autoZoom + 'body {padding: 1em!important; max-width: 90%!important; overflow-wrap: break-word; word-wrap: break-word;} img {max-width: 85%!important;} a {color: inherit !important;} .inline-comment {border-left: .4em solid #F2D400!important; background-color: #F2E6AA!important; border-radius: 2px; padding: .2em; margin-left: 0em; margin-right: .1em;} .block-comment { font-size: 1em!important; background-color: #F2E6AA!important;  border-radius: 3px; padding: .5em 1.5em; transform: rotate(' + rotate + 'deg); margin: 1.5em; border-top: .5em solid #F2D400!important; }' + noLinks + pageBreaks + '</style>';
		// wmColorsCSS = wmColorsCSS.replace(/\[_bck_color_\]/g, bcHex).replace(/\[_txt_color_\]/g, fcHex);
	}


	

	function getTextToShow() {
		if (monkey._editor.somethingSelected()) {
			return monkey._editor.getSelection();
		} else {
			var t = monkey._editor.getFocusedBlock();
			if (t != false) {
				return t.text;
			} else {
				return monkey._editor.getValue().substring(monkey.storage.getMetaFields().endPoint);
			}
		}
	}


	function updatePreview() {

		var iframe = $iframe[0];
		var doc = iframe.document;
		
		
		if (iframe.contentDocument) {
			doc = iframe.contentDocument;
		} else if (iframe.contentWindow) {
			doc = iframe.contentWindow.document;
		}
		doc.open();
		doc.writeln(getHtml(getTextToShow()));
		doc.close();

		

		doc.addEventListener('contextmenu', function(e) {
			e.preventDefault();
			var pos = $("#preview-iframe").position();
			getContextMenu().popup(e.x + parseInt(pos.left), e.y + parseInt(pos.top));
		}, true);

		doc.addEventListener('dblclick', function (e) {
			e.preventDefault();
			updatePreview();
		}, true);

	}



	function print() {
		updatePreview(); // so last version is printed even if pane is 
		// $iframe[0].contentWindow.document.body.style.fontSize = "1.1em";
		$iframe[0].contentWindow.print();
	}


	function getHtml(text) {


		var content = handleComments(text);
		content = handleForcedLines(content);
		content = removeToDos(content);
		// content = window.marked(content);
		content = window.mdIt.render(content);
		

		var num = monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] && monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] == "1" ? numberedHeadingsCSS: "";

		
		//var disableLinks = '\n<script>var anchors = document.getElementsByTagName("a"); for (var i = 0; i < anchors.length; i++) { anchors[i].onclick = function() {return(false);};}</script>\n';

		// font-size:4vw!important;
		return '<!DOCTYPE html><html>\n<head>\n<meta http-equiv="Content-Type" content="text/html; charset=utf-8">\n<script>\n' + templateJS + '\n</script>\n<style type="text/css">\n' + templateCSS + '</style>\n' + wmColorsCSS + num + '\n<title>' + (monkey._currentDoc && monkey._currentDoc.nam ? monkey._currentDoc.nam.substring(0, 20) : 'WM export') + '</title>\n   <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/katex@0.10.0/dist/katex.min.css" integrity="sha384-9eLZqc9ds8eNjO3TmqPeYcDj8n+Qfa4nuSiGYa6DjLNcv9BtN69ZIulL9+8CqC9Y" crossorigin="anonymous"><script defer src="https://cdn.jsdelivr.net/npm/katex@0.10.0/dist/katex.min.js" integrity="sha384-K3vbOmF2BtaVai+Qk37uypf7VrgBubhQreNQe9aGsz9lB63dIFiQVlJbr92dw2Lx" crossorigin="anonymous"></script><script defer src="https://cdn.jsdelivr.net/npm/katex@0.10.0/dist/contrib/auto-render.min.js" integrity="sha384-kmZOZB5ObwgQnS/DuDg6TScgOiWWBiVt0plIRkZCmE6rDZGrEOQeHM5PcHi+nyqe" crossorigin="anonymous" onload=\'renderMathInElement(document.body, {throwOnError: false, strict: false, displayMode: false, delimiters: [{left: "$$$", right: "$$$", display: true}, {left: "$$", right: "$$", display: false}]});\'></script>\n </head>\n<body style="font-size:.8em;">\n<div id="wrapper">\n' + content + '\n</div></body>\n</html>';
	}

	// delimiters: [{left: "$$", right: "$$", display: true}, {left: "$", right: "$", display: false}]

	//  {left: "\(", right: "\)", display: false}, {left: "\\[", right: "\\]", display: true}
	

	function saveTempAndExport(ext) {
		updatePreview(); // so last version is printed even if pane is hidden
		setColors(true);
		var tempPath = monkey.os.tmpdir() + "/wm3export." + ext;
		var html = getHtml(getTextToShow());
		
		monkey.fs.writeFile(tempPath, html, function(error) {
			monkey.opener(tempPath);
			setColors(false);
		});

	}

	function handleComments(text) {

		text = text.replace(/[^:]\/{2}(.*?)\/{2}/gm, monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1" ? '<span class="inline-comment">$1</span>' : "");

		text = text.replace(/^\/{2}(.*?)$/gm, monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1" ? '<div class="block-comment">$1</div>\n\n' : "");

		//text = text.replace(/(?!^)[^:]\/{2}\s*(.*?)$/gm, monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1" ? '<span class="inline-comment">$1</span>' : "");

		return text;
	}

	function removeToDos(text) {
		return text.replace(/^-\s\[[x| ]\].+$/gm, "");
	}


	function handleForcedLines(text) {
		if (monkey.localStorage["EXPORT_FORCED_NEW_LINES"] == "1") {
			return text.replace(/^\ $/gm, "&nbsp;\n");
		} else {
			return text;
		}
	}


	function copyCleanHtml() {

		var text = monkey._editor.somethingSelected() ? monkey._editor.getSelection() : monkey._editor.getValue();

		if (!monkey._editor.somethingSelected()) {
			text = text.substring(monkey.storage.getMetaFields().endPoint, text.length);	
		}


		var content = handleComments(text);
		content = handleForcedLines(content);
		content = removeToDos(content);
		content = window.mdIt.render(content);
		

		var clipboard = monkey.gui.Clipboard.get();
		
		if (!monkey._editor.somethingSelected() && options.clean_html_get_full === true) {
			clipboard.set('<!DOCTYPE html>\n<html>\n<head>\n<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />\n<meta name="description" content="This file was exported from WriteMonkey editor.">\n<title>' + monkey._currentDoc.nam.substring(0, 100) + '</title>\n</head>\n<body>\n' + content + '\n</body>\n</html>', 'text');
		} else {
			clipboard.set(content, 'text');
		}

		

		monkey.wm.showNotice('HTML copied', {autoClose: true});
	}



	/*
	########   ##     ##  ########   ##    ##   ########    ######    
	##         ##     ##  ##         ###   ##      ##      ##    ##   
	##         ##     ##  ##         ####  ##      ##      ##         
	######     ##     ##  ######     ## ## ##      ##       ######    
	##          ##   ##   ##         ##  ####      ##            ##   
	##           ## ##    ##         ##   ###      ##      ##    ##   
	########      ###     ########   ##    ##      ##       ######    
	*/



	// on change
	var lastUpdateLines = 0;
	monkey._editor.on("change", function(doc) {

		if (!monkey.universalPane.isPaneVisible("wm_preview")) 
			return;


		var lineCount = monkey._editor.lineCount();
		if (lastUpdateLines != lineCount) {
			setTimeout(function() { updatePreview(); lastUpdateLines = lineCount; }, 100);
		}
	});


	monkey.document.body.addEventListener("switchPaneE", function(e) {
		if (e.detail.newId == "wm_preview") {
			updatePreview();
		} else {
			
		}
	}, false);


	// document load / swap
	monkey._editor.on("swapDoc", function(sevent) {
		if (monkey.universalPane.isPaneVisible("wm_preview"))
			setTimeout(function() { updatePreview(); }, 100);
	});


	// update on save
	monkey.document.body.addEventListener("saveE", function() {
		setTimeout(function() {
			updatePreview();
		}, 1);
	}, false);

	// update on focus
	monkey.document.body.addEventListener("focusE", function() {
		setTimeout(function() {
			updatePreview();
		}, 1);
	}, false);


	// scroll
	monkey._editor.on("scroll", function(sevent) {

		if (!monkey.universalPane.isPaneVisible("wm_preview") || monkey.localStorage["EXPORT_SYNC_SCROLLING"] != "1")
			return;


		var originPos = monkey._editor.doc.scrollTop;
		var originHeight = monkey._editor.getScrollerElement().scrollHeight; //monkey._editor.doc.height; // monkey._editor.getScrollInfo().height;
		var targetHeight = $iframe[0].contentWindow.document.body.scrollHeight;
		var offset = 0;

		var origPercent = (originPos - offset) / originHeight;
		var targetPos = targetHeight * origPercent;

		$iframe[0].contentWindow.scrollTo(0, targetPos);

	});

	
	return {
		_init: _init,
		updatePreview: updatePreview,
		print: print,
		copyCleanHtml: copyCleanHtml,
		saveTempAndExport: saveTempAndExport,
		getTemplatePaletteItems: getTemplatePaletteItems,
		changeTemplate: changeTemplate
	};

};